package celery.engine;

import celery.engine.updatehandlers.DrawHandlerSettings;
import celery.engine.updatehandlers.PhysicsHandlerSettings;

/**
 * EngineSettings controls what parts of the Engine are instantiated, and what
 * are not, as well as the control of flow of the Engine and what features are
 * enabled and what are not. You should try not to enable features which are not
 * going to be used, since this will save resources, but keep in mind that
 * EngineSettings remain constant throughout the lifetime of the Engine, and
 * cannot be changed after initialization.
 * 
 * @author psastras
 * 
 */
public class EngineSettings {

	public EngineSettings() {
		drawSettings = new DrawHandlerSettings();
		physicsSettings = new PhysicsHandlerSettings();
	}

	// / Settings controlling the drawing
	public DrawHandlerSettings drawSettings;
	// / Physics settings controls settings of the physics engine
	public PhysicsHandlerSettings physicsSettings;
	// / True if the Engine should automatically register a TouchInputHandler
	public boolean useTouchInput;
	// / True if the Engine should automatically register a KeyboardInputHandler
	public boolean useKeyboardInput;
}
