package celery.engine.graphics;

import celery.engine.Engine;

public abstract class Animation {

	protected long prevTime_;
	protected boolean isRunning_;
	protected boolean isFinished_;
	protected SceneEntity entity_;

	public Animation(SceneEntity e) {
		entity_ = e;
		prevTime_ = 0;
		isRunning_ = false;
	}

	public boolean isRunning() {
		return isRunning_;
	}

	public void start() {
		prevTime_ = Engine.instance().elapsedTime();
		isRunning_ = true;
	}

	public void runTimeStep() {
		prevTime_ = Engine.instance().elapsedTime();
		if (entity_.isMarkedForRemoval())
			this.stop();
	}

	public void stop() {
		isFinished_ = true;
		isRunning_ = false;
		this.onAnimationFinished();
	}

	protected abstract void onAnimationFinished();

	public abstract void reset(boolean loop);

	public boolean isFinished() {
		return isFinished_;
	}

	protected long elapsedTime() {
		return Engine.instance().elapsedTime() - prevTime_;
	}

}
