package celery.engine.graphics;

import celery.engine.utils.EngineConstants;
import celery.engine.utils.math.Vec2F;

/**
 * Viewport describes a origin, width, and height of a scene viewport in virtual
 * coordinates. Objects drawn using a viewport will automatically be scaled to
 * best fit of the actual screen size.
 * 
 * @author psastras
 * 
 */

public class Viewport {

	protected Vec2F pos_, bounds_;
	protected Vec2F screenSize_;
	protected Vec2F offset_ = new Vec2F();
	protected Vec2F maxVirtual_;
	protected float ratio_;

	public Viewport(Vec2F pos, Vec2F bounds) {
		pos_ = pos;
		bounds_ = bounds;
	}

	public static final Viewport defaultViewport() {
		return new Viewport(EngineConstants.Zero2F, new Vec2F(3.f, 2.f));
	}

	public final Vec2F pos() {
		return pos_;
	}

	public final Vec2F screenSize() {
		return screenSize_;
	}

	public final Vec2F bounds() {
		return bounds_;
	}

	public final float ratio() {
		return ratio_;
	}

	public final Vec2F offset() {
		return offset_;
	}

	Vec2F tmp_ = new Vec2F();

	public void updateScreenSize(Vec2F screen) {
		if (screenSize_ == null || !screen.isEqual(screenSize_)) {
			screenSize_ = screen;

			ratio_ = tmp_.set(screen).divEq(bounds_).min();
			offset_.set(screen).subEq(tmp_.set(bounds_).scaleEq(ratio_))
					.scaleEq(0.5f);

		}
	}

	public boolean wrapVirtual(Vec2F virtual) {
		if (virtual.x > bounds_.x) {
			virtual.x -= bounds_.x;
			return true;
		} else if (virtual.x < pos_.x) {
			virtual.x += bounds_.x;
			return true;
		}
		if (virtual.y > bounds_.y) {
			virtual.y -= bounds_.y;
			return true;
		} else if (virtual.y < pos_.y) {
			virtual.y += bounds_.y;
			return true;
		}
		return false;
	}

	public final Vec2F virtualToScreen(final Vec2F coords) {
		tmp_.set(coords).addEq(pos_).scaleEq(ratio_).addEq(offset_);
		return tmp_;
	}

	Vec2F tmp2_ = new Vec2F();

	public final Vec2F screenToVirtual(final Vec2F coords) {
		return tmp_.set(coords).scaleEq(1.f / ratio_).subEq(pos_)
				.subEq(tmp2_.set(offset_).scaleEq(1.f / ratio_));

	}
}
