package celery.engine.graphics.animation;

import celery.engine.Engine;
import celery.engine.graphics.Animation;
import celery.engine.graphics.SceneEntity;
import celery.engine.utils.math.Vec2F;

public class SlideAnimation extends Animation {

	protected Vec2F finalPosition_;
	protected long elapsedTime_;
	protected long animationTime_;
	protected Vec2F direction_;

	protected int type_ = 0;
	protected boolean wrap_ = false;

	public SlideAnimation(SceneEntity e, Vec2F end, int animationTime) {
		super(e);
		finalPosition_ = end;
		elapsedTime_ = 0;
		animationTime_ = animationTime;
		direction_ = finalPosition_.sub(entity_.transform().translate);
		type_ = 0;
	}

	public SlideAnimation(SceneEntity e, Vec2F end, Vec2F dir) {
		super(e);
		finalPosition_ = end;
		direction_ = dir;
		elapsedTime_ = 0;
		direction_ = finalPosition_.sub(entity_.transform().translate)
				.normalize().scaleEq(dir.length());
		type_ = 1;
	}

	public SlideAnimation(SceneEntity e, Vec2F dir, boolean wrap) {
		super(e);
		direction_ = new Vec2F(dir);
		elapsedTime_ = 0;
		finalPosition_ = new Vec2F();
		type_ = 2;
		wrap_ = wrap;
	}

	public final void setDirection(Vec2F dir) {
		direction_.set(dir);
	}

	public final Vec2F dir() {
		return direction_;
	}

	@Override
	public void reset(boolean loop) {
		// TODO Auto-generated method stub

	}

	@Override
	protected void onAnimationFinished() {
		// TODO Auto-generated method stub

	}

	@Override
	public void runTimeStep() {
		if (!isRunning_)
			return;
		long dt = Engine.instance().elapsedTime() - prevTime_;
		if (type_ == 2) {
			finalPosition_.set(direction_);// .scaleEq(dt / 1000.f);
			entity_.transform().translate.addEq(finalPosition_);
			if (Engine.instance().drawHandler().viewport()
					.wrapVirtual(entity_.transform().translate)
					&& !wrap_) {
				this.stop();
			}
		} else if (type_ == 1) {
			float d1 = entity_.transform().translate.distance2(finalPosition_);
			entity_.transform().translate.addEq(direction_.scale(dt / 1000.f));
			float d2 = entity_.transform().translate.distance2(finalPosition_);
			if (d2 >= d1) {
				this.stop();
			}
		} else {
			entity_.transform().translate.addEq(direction_.scale(dt
					/ (float) animationTime_));
			elapsedTime_ += dt;
			if (elapsedTime_ >= animationTime_) {
				entity_.transform().translate = finalPosition_;
				this.stop();
			}
		}

		super.runTimeStep();
	}
}
