package celery.engine.graphics.animation;

import android.graphics.Rect;
import celery.engine.graphics.Animation;
import celery.engine.graphics.entities.Sprite;

public class SpriteAnimation extends Animation {

	protected Sprite sprite_;
	protected Rect frameBounds_;
	protected int nFrames_;
	protected float fps_;
	protected Rect curFrameBounds_ = new Rect();
	protected boolean loop_, isFinished_;
	protected long startTime_;

	public SpriteAnimation(Sprite sp, int nFrames, float fps, boolean loop) {
		super(sp);
		sprite_ = sp;
		loop_ = loop;
		isFinished_ = false;
		frameBounds_ = new Rect(0, 0, sp.bitmap().getWidth() / nFrames, sp
				.bitmap().getHeight());
		nFrames_ = nFrames;
		fps_ = fps;
		sprite_.setSrcRect(frameBounds_);
	}

	@Override
	public void runTimeStep() {
		if (!isRunning_)
			return;

		if ((this.elapsedTime() / (1000 / fps_)) > nFrames_) {
			isFinished_ = true;
			this.onAnimationFinished();
			return;
		}

		int dt = (int) ((this.elapsedTime() / (1000 / fps_)) % nFrames_) + 1;
		curFrameBounds_.set((dt - 1) * frameBounds_.right, 0, dt
				* frameBounds_.right, frameBounds_.bottom);
		sprite_.setSrcRect(curFrameBounds_);
		// System.out.println(curFrameBounds_.width() + " x "
		// + curFrameBounds_.height());
	}

	@Override
	public void reset(boolean loop) {
		isFinished_ = !loop;
		isRunning_ = false;
		curFrameBounds_.set(0, 0, frameBounds_.right, frameBounds_.bottom);
		sprite_.setSrcRect(curFrameBounds_);
	}

	@Override
	public boolean isFinished() {
		return isFinished_;
	}

	@Override
	protected void onAnimationFinished() {
		// TODO Auto-generated method stub

	}

}
