package celery.engine.graphics.entities;

import android.graphics.Canvas;
import celery.engine.Engine;
import celery.engine.graphics.SceneMaterial;
import celery.engine.graphics.Viewport;
import celery.engine.inputhandlers.TouchInputHandler;
import celery.engine.utils.Utils;
import celery.engine.utils.math.Transform;
import celery.engine.utils.math.Vec2F;
import celery.engine.utils.math.Vec4F;

public class Rectangle extends Polygon {

	/**
	 * Constructs a drawable rectangle with the given bounds (in virtual space),
	 * which can be then added to a scene.
	 * 
	 * @param bounds
	 */

	public Rectangle(Transform tfo, SceneMaterial sm, TouchInputHandler hdl) {
		super(tfo, sm, hdl);

		Vec2F[][] pointList = new Vec2F[1][];
		pointList[0] = new Vec2F[4];
		pointList[0][0] = new Vec2F(-0.5f, -0.5f);
		pointList[0][1] = new Vec2F(0.5f, -0.5f);
		pointList[0][2] = new Vec2F(0.5f, 0.5f);
		pointList[0][3] = new Vec2F(-0.5f, 0.5f);
		geoDataModel_ = new GeometryData(pointList, null);
		geoDataWorld_ = new GeometryData(pointList, null);
	}

	@Override
	public Vec4F getAABB() {
		return new Vec4F(transfo_.translate.x, transfo_.translate.y,
				transfo_.scale.x, transfo_.scale.y);
	}

	@Override
	public boolean isInBounds(Vec2F coord) {
		final Vec2F model = Utils.toModelSpace(transfo_, coord);
		// System.out.println(coord + "-->" + model);
		return model.x >= -0.5f && model.y >= -0.5f && model.x <= 0.5f
				&& model.y <= 0.5f;
	}

	@Override
	public void draw(Canvas c, Engine e, Viewport v, Vec4F bounds) {
		c.drawRect(-0.5f, -0.5f, 0.5f, 0.5f, material_.paint);
	}

	@Override
	public GeometryData geometryData() {
		return geoDataModel_.transform(transfo_, geoDataWorld_);
	}
}
