package celery.engine.updatehandlers;

import java.util.HashSet;
import java.util.Iterator;

import android.content.Context;
import android.os.Message;
import android.os.SystemClock;
import android.util.FloatMath;
import celery.engine.Engine;
import celery.engine.graphics.entities.PhysicsEntity;
import celery.engine.utils.MessageHandler;
import celery.engine.utils.math.Vec2F;

public class PhysicsHandler implements UpdateHandler {
	protected PhysicsHandlerSettings settings_;
	protected long mSecUPSWait;
	protected boolean isRunning_ = false;
	protected HashSet<PhysicsEntity> data_ = new HashSet<PhysicsEntity>();
	protected long prevTime_ = 0;
	protected long remainderTime_ = 0;

	public PhysicsHandler(PhysicsHandlerSettings settings, Context context) {
		settings_ = settings;
		mSecUPSWait = (int) (1000.f / settings.ups);
	}

	public void remove(PhysicsEntity e) {
		data_.remove(e);
	}

	public void clear() {
		data_.clear();
	}

	protected void update() {
		long elapsedtime = elapsedTime();
		if (remainderTime_ >= settings_.k) {
			elapsedtime += remainderTime_;
			remainderTime_ = 0;
		}
		final int updates = (int) FloatMath.floor(elapsedtime / settings_.k);
		for (int i = 0; i < updates; i++) { // reun vertlet
			Iterator<PhysicsEntity> it = data_.iterator();
			while (it.hasNext()) {
				PhysicsEntity e = it.next();
				if (e.entity.isMarkedForRemoval()) {
					data_.remove(e);
				} else {
					integrate(e, elapsedtime / updates / 1000.f);
				}
			}

			resolveConstraints(updates);
			// apply constraints
		}

		remainderTime_ += elapsedtime - (updates * settings_.k);
	}

	public void resolveConstraints(int updates) {
		int iters = 0;
		boolean allConstraintsSatisfied = false;
		while (!allConstraintsSatisfied && iters++ < 15) {
			Iterator<PhysicsEntity> cit = data_.iterator();
			boolean allsat = true;
			while (cit.hasNext()) {
				PhysicsEntity e = cit.next();
				final int nConstraints = e.constraints.size();
				for (int j = 0; j < nConstraints; j++) {
					boolean success = e.constraints.get(j).applyConstraint(e,
							(updates * settings_.k) / 1000.f);
					if (!success) {
						allsat = false;
					}
				}
				// e.currentPosition = new Vec2F(
				// e.entity.transform().translate);
			}
			allConstraintsSatisfied = allsat;

		}
	}

	protected Vec2F temp_ = new Vec2F();
	protected Vec2F temp_2 = new Vec2F();

	protected void integrate(final PhysicsEntity e, final float dt) {

		// if (!e.currentPosition.isEqual(e.entity.transform().translate)) {
		// Vec2F dif = e.currentPosition.sub(e.previousPosition);
		// e.currentPosition = new Vec2F(e.entity.transform().translate);
		// e.previousPosition = e.currentPosition.sub(dif);
		// }
		final float alpha = 0.001f;
		temp_.set(e.entity.transform().translate);
		e.entity.transform().translate.scaleEq(2.f - alpha)
				.subEq(temp_2.set(e.previousPosition).scaleEq(1.f - alpha))
				.addEq(temp_2.set(e.accelAccum).scaleEq(dt * dt));
		e.previousPosition.set(temp_);
		e.accelAccum.set(e.acceleration);
		// e.currentPosition = new Vec2F(e.entity.transform().translate);
	}

	protected long elapsedTime() {
		return Engine.instance().elapsedTime() - prevTime_;
	}

	@Override
	public void pause() {
		isRunning_ = false;
	}

	public void addItem(PhysicsEntity e) {
		data_.add(e);
	}

	@Override
	public void start() {
		isRunning_ = true;
		prevTime_ = Engine.instance().elapsedTime();
		this.sendUpdateMessage();

	}

	@Override
	public void resume() {
		isRunning_ = true;
		this.sendUpdateMessage();

	}

	@Override
	public void receiveMessage(Message msg) {
		if (isRunning_) {
			this.update();
		}
		prevTime_ = Engine.instance().elapsedTime();
		this.sendUpdateMessage();

	}

	@Override
	public void stop() {
		isRunning_ = false;

	}

	protected void sendUpdateMessage() {
		MessageHandler h = Engine.instance().messageHandler();
		h.sendMessageAtTime(this, h.obtainMessage(), SystemClock.uptimeMillis()
				+ mSecUPSWait);
	}

}
