package celery.engine.utils.math;

/**
 * Defines a generic 3D vector and useful operations.
 * 
 * @author psastras
 * 
 */

public final class Vec3F {

	public float x, y, z;

	public Vec3F(float d0, float d1, float d2) {
		x = d0;
		y = d1;
		z = d2;
	}

	public Vec3F() {
		x = 0.f;
		y = 0.f;
		z = 0.f;
	}

	public Vec3F(Vec3F v2f) {
		x = v2f.x;
		y = v2f.y;
		z = v2f.z;
	}

	public final Vec3F add(Vec3F rhs) {
		return new Vec3F(this).addEq(rhs);
	}

	public final Vec2F add(Vec2F rhs) {
		return new Vec2F(this).addEq(rhs);
	}

	public final Vec3F addEq(Vec3F rhs) {
		x += rhs.x;
		y += rhs.y;
		z += rhs.z;
		return this;
	}

	public final Vec3F sub(Vec3F rhs) {
		return new Vec3F(this).subEq(rhs);
	}

	public final Vec2F sub(Vec2F rhs) {
		return new Vec2F(x - rhs.x, y - rhs.y);
	}

	public final Vec3F subEq(Vec3F rhs) {
		x -= rhs.x;
		y -= rhs.y;
		z -= rhs.z;
		return this;
	}

	public final Vec3F mul(Vec3F rhs) {
		return new Vec3F(x * rhs.x, y * rhs.y, z * rhs.z);
	}

	public final Vec3F mulEq(Vec3F rhs) {
		x *= rhs.x;
		y *= rhs.y;
		z *= rhs.z;
		return this;
	}

	public final Vec3F scale(float rhs) {
		return new Vec3F(rhs * x, rhs * y, rhs * z);
	}

	public final Vec3F scaleEq(float rhs) {
		x *= rhs;
		y *= rhs;
		z *= rhs;
		return this;
	}

	public final Vec3F div(float rhs) {
		return new Vec3F(this).divEq(rhs);
	}

	public final Vec3F divEq(float rhs) {
		x /= rhs;
		y /= rhs;
		z /= rhs;
		return this;
	}

	public final float dot(Vec3F rhs) {
		return x * rhs.x + y * rhs.y + z * rhs.z;
	}

	public final float length2() {
		return x * x + y * y + z * z;
	}

	public final float length() {
		return (float) Math.sqrt(this.length2());
	}

	@Override
	public String toString() {
		return "[" + x + ", " + y + ", " + z + "]";
	}
}
