package celery.engine.utils.math;

import android.graphics.Rect;
import android.graphics.RectF;

/**
 * Defines a generic 3D vector and useful operations.
 * 
 * @author psastras
 * 
 */

public final class Vec4F {

	public float x, y, z, w;

	public Vec4F(float d0, float d1, float d2, float d3) {
		x = d0;
		y = d1;
		z = d2;
		w = d3;
	}

	public Vec4F() {
		x = 0.f;
		y = 0.f;
		z = 0.f;
		w = 0.f;
	}

	public Vec4F(Vec4F v4f) {
		x = v4f.x;
		y = v4f.y;
		z = v4f.z;
		w = v4f.w;
	}

	public final Vec4F add(Vec4F rhs) {
		return new Vec4F(rhs.x + x, rhs.y + y, rhs.z + z, rhs.w + w);
	}

	public final Vec4F addEq(Vec4F rhs) {
		x += rhs.x;
		y += rhs.y;
		z += rhs.z;
		w += rhs.w;
		return this;
	}

	public final Vec4F sub(Vec4F rhs) {
		return new Vec4F(rhs.x - x, rhs.y - y, rhs.z - z, rhs.w - w);
	}

	public final Vec4F subEq(Vec4F rhs) {
		x -= rhs.x;
		y -= rhs.y;
		z -= rhs.z;
		return this;
	}

	public final Vec4F mul(Vec4F rhs) {
		return new Vec4F(x * rhs.x, y * rhs.y, z * rhs.z, w * rhs.w);
	}

	public final Vec4F mulEq(Vec4F rhs) {
		x *= rhs.x;
		y *= rhs.y;
		z *= rhs.z;
		w *= rhs.w;
		return this;
	}

	public final Vec4F scale(float rhs) {
		return new Vec4F(rhs * x, rhs * y, rhs * z, rhs * w);
	}

	public final Vec4F scaleEq(float rhs) {
		x *= rhs;
		y *= rhs;
		z *= rhs;
		w *= rhs;
		return this;
	}

	public final Vec4F div(float rhs) {
		return new Vec4F(rhs / x, rhs / y, rhs / z, rhs / w);
	}

	public final Vec4F divEq(float rhs) {
		x /= rhs;
		y /= rhs;
		z /= rhs;
		w /= rhs;
		return this;
	}

	public final float dot(Vec4F rhs) {
		return x * rhs.x + y * rhs.y + z * rhs.z + w * rhs.w;
	}

	public final float length2() {
		return x * x + y * y + z * z + w * w;
	}

	public final float length() {
		return (float) Math.sqrt(this.length2());
	}

	public final RectF toRectF() {
		return new RectF(x, y, z, w);
	}

	public final Rect toRect() {
		return new Rect((int) x, (int) y, (int) z, (int) w);
	}
}
